---
title: "Tutoriel git"
output:
    toc: true
---

* * * * *

## Introduction

### À qui s'adresse ce tutoriel ?
- Objectif du tutoriel : apprendre à utiliser le système git et le [serveur
  gogs](https://gogs.univ-littoral.fr) mis en place par le
[SCOSI](http://scosi.univ-littoral.fr/). 
- Pré-requis : utilisation basique d'un ordinateur.
- Public visé : les développeurs au sens large (code latex, python, matlab, R,
  java...)

### Qu'est-ce-que git et gogs ?
- git : système de journalisation de code source décentralisé (alternative à
  mercurial, subversion, CVS...)
- gogs : service d'hébergement de code source utilisable avec git (alternative
  à github, bitbucket...)

### Dans quels cas utiliser git/gogs ?
git est conçu pour manipuler principalement des fichiers **au format texte**
(code source, code latex, fichier CSV...). Il permet notamment :

- journalisation 
- sauvegarde distante 
- synchronisation 
- travail en équipe 
- projets publics ou privés :

### Dans quels cas ne pas utiliser git/gogs ?
- édition collaborative en temps-réel -> sharelatex
- partage de fichiers -> serveur ftp, http...
- fichiers dans un format non textuel (jpg, word, excel...) -> dropbox

### Concepts de base

#### Notion de "version" (commit)

Un "projet git" contient l'historique de toutes les modifications de chaque
fichier.  On appelle "commit" un état sauvegardé de ces modifications (une
version du projet en quelque sorte).

Généralement, on modifie le projet à partir de la version précédente et ansi de
suite; ce qui correspond à une succession de commits :

![](concepts_commits_1.svg)

Mais on peut également effectuer des modifications en parallèles. On appelle ça
des branches :

![](concepts_commits_2.svg)

On peut ensuite rassembler ces modifications en fusionnant les branches :

![](concepts_commits_3.svg)

#### Notion de dépôt (repository)

On appelle dépôt l'ensemble des fichiers de notre "projet git", avec
l'historique des commits.

Un dépôt peut être dupliqué sur différentes machines, pour synchroniser les
données ou pour travailler en équipe. 

Dans l'ancien temps des années 2000, on avait un dépôt principal distant que
l'on synchronisait ou modifiait via des copies locales partielles (subversion,
CVS...) :

<img src="concepts_depots_svn.svg" style="width:700px">

Aujourd'hui, on utilise plutôt un système de dépôts décentralisés (git,
mercurial...).  Avec ce système, tous les dépôts sont identiques et peuvent se
synchroniser entre eux :

<img src="concepts_depots_git.svg" style="width:700px">

Généralement, on garde tout de même un dépôt sur un serveur distant accessible
en permanence (serveur gogs, github...).


* * * * *

## Installation et configuration

Pour utiliser git, il vous faut une connexion internet et un client git.

### Installation sous Linux

La plupart des systèmes Linux modernes propose le client git dans leur système
de paquets. Par exemple, sur les Linux basés Debian, il suffit de lancer la
commande console :

```
sudo apt-get install git
```

Il existe également des clients graphiques : gitg, giggle, qgit, gitk, git-gui...

### Installation sous Windows ou Mac

Télécharger et installer le client [git pour
Windows](http://git-scm.com/download/win) ou le client [git pour
Mac](http://git-scm.com/download/mac).

### Configuration

Après avoir installé le client git, il faut configurer le nom et l'adresse
e-mail de l'utilisateur (à faire une seule). Par exemple, en tapant les
commandes consoles : 

```
$ git config --global user.name "Julien Dehos"
$ git config --global user.email dehos@nimpe.org
```

On peut également configurer l'éditeur de texte et le proxy que le client git
doit utiliser. Par exemple, en tapant les commandes consoles :

```
$ git config --global core.editor emacs
$ git config --global http.proxy http://user:pass@proxyhost:proxyport
```
ou alors en ajoutant des variables d'environnement (par exemple, dans le
`.bashrc`): 
```
export EDITOR=emacs
export http_proxy=http://user:pass@proxyhost:proxyport
```

### Obtenir de l'aide

Aide générale sur git :
```
$ git help 
```

Aide sur une commande git :
```
$ git help <commande git>
```

Voir également la section [références](#références).

### Exercice

Installez et configurez le client git sur votre machine.  Vérifiez que votre
client git peut accéder à l'extérieur, par exemple en récupérant le dépôt
distant `https://github.com/juliendehos/invinoveritas` :

![](installation_01.png)


* * * * *

## Dépôt local (journaliser un projet)

L'élément de base d'un projet git est le dépôt. Il s'agit simplement d'un
dossier classique que l'on demande à git de versionner.

### Initialiser un dépôt

Allez dans le dossier à versionner puis lancez la commande `git init`. 

![](depot_local_01.png)

Git créé alors un dossier `.git` pour y stocker ses metadonnées.  Si vous ne
voulez plus versionner votre dossier, il suffit de supprimer ce dossier.  Le
reste du temps, ne touchez pas à ce dossier si vous n'êtes pas absolument
sûr(e) de ce que vous faites.

### Afficher l'état d'un dépôt

La commande `git status` affiche l'état courant du dépôt : nouveaux fichiers,
fichiers modifiés, fichiers supprimés.  Cette commande est très sûre; elle ne
peut rien casser.  N'hésitez donc pas à la lancer pour voir où vous en êtes.

![](depot_local_02.png)

### Ajouter des fichiers dans le dépôt

Il faut indiquer explicitement les fichiers que git doit journaliser.  

Par exemple, imaginons qu'on ouvre un nouveau fichier `rapport_01.tex` dans
notre projet.

![](depot_local_03.png)

On y écrit du code latex.

![](depot_local_04.png)

Et on affiche l'état du dépôt.

![](depot_local_05.png)

Git nous indique que le fichier `rapport_01.tex` existe bien dans le dossier
(ainsi qu'un fichier de sauvegarde), mais qu'il ne fait pas partie du dépôt. 

Pour ajouter un fichier au dépôt, il faut utiliser la commande `git add`. 

![](depot_local_06.png)

Ceci indique que git doit prendre en compte le fichier lors des prochains
commits (journalisation), ce qu'on peut vérifier avec un `git status`.

![](depot_local_07.png)

### Valider les modifications (commit)

Après avoir ajouté, modifié ou supprimé des fichiers, on peut valider les
modifications, c'est-à-dire sauvegarder un état du projet (commit). Encore une
fois, seuls les fichiers qui ont été ajoutés au projet (avec `git add`) sont
pris en compte. 

La commande à lancer pour valider toutes les modifications est `git commit -a`.

![](depot_local_08.png)

Git lance alors l'éditeur de texte pour le message de commit.
Le message est pré-rempli avec des informations concernant le commit. Ces
informations vous permettent de vérifier que le commit correspond bien à ce que
vous voulez mais ne feront pas partie du message final.

Écrivez un message expliquant rapidement les modifications que vous êtes en
train de valider puis enregistrez et quittez. 

![](depot_local_09.png)

Git créé alors le nouveau commit, qui valide ici le fichier `rapport_01.tex`.

![](depot_local_10.png)

Vous pouvez le vérifier avec un `git status`.

![](depot_local_11.png)

### Fichiers autogénérés et .gitignore

Git est très pratique pour gérer des fichiers textes que l'on modifie
régulièrement. En revanche, il n'est pas adapté pour gérer des fichiers dans un
format non-textuel (images, données compressées, documents word...).  Si on
ajoute un tel fichier dans le dépôt alors à chaque modification, git stockera
une nouvelle copie complète (au lieu de stocker uniquement les lignes qui ont
changé), ce qui est inefficace.

D'autre part, il est inutile de versionner les fichiers autogénérés (créés
automatiquement lors de la compilation du code). Par exemple, si on compile le
code latex de notre projet : 

![](depot_local_12.png)

des fichiers intermédiaires sont créés et apparaissent lors des `git status` :

![](depot_local_13.png)

Pour les ignorer, il suffit d'ajouter un fichier `.gitignore` et d'y indiquer
les fichiers à ignorer.

![](depot_local_14.png)

Ainsi, il n'apparaissent plus lors des `git status`.

![](depot_local_15.png)

Il est de pratique courante d'ajouter le `.gitignore` au dépôt, ainsi qu'un
fichier `README.md` qui servira de page d'accueil (au format
[markdown](https://github.com/adam-p/markdown-here/wiki/Markdown-Cheatsheet)).

![](depot_local_16.png)


### Renommer et supprimer des fichiers

La commande `git mv` permet de renommer un fichier versionné. Il ne faut pas
utiliser la commande système (`mv`) car dans ce cas git ne saura pas qu'il faut
renommer le fichier (il pensera que le fichier manque et essaiera de le
recréer).

![](depot_local_17.png)

De même, la commande `git rm` permet de supprimer un fichier versionné.

![](depot_local_18.png)


### Afficher l'historique des commits

Faisons d'abord une modification "commitée" puis une autre non commitée.

![](depot_local_19.png)

La commande `git log` permet d'afficher l'historique des commits.

![](depot_local_20.png)

On peut également le voir avec un client graphique graphique comme `gitg`.

![](depot_local_21.png)

Le "unstaged" correspond aux modifications courantes non commitées.

![](depot_local_log.svg)

a8...
ff...
07...
7b...
59...

a8b24fd52f55495b07a486a16f684dab25789285
ff07d510a081863053d27d4107a0da0039c2d2e0
073437274cb37c7d50afbed20341b76984a8a8a6
7b3e3b9d501892bdbbfd4c7592221535d9c16399
59d966459ea11b56464bfa0b23ea85465b3c0bc4


### Supprimer les modifications courantes

### Supprimer des commits précédents

### Revenir à un commit précédent (sans suppression)


TODO


### Exercice



* * * * *

## Dépôt distant (sauvegarder un projet sur un serveur)

TODO

nouveau projet

projet existant

### Exercice

* * * * *

## Branches (travailler en équipe)

TODO

### Exercice

* * * * *

## Forks (collaborer avec d'autres projets)

### Exercice

* * * * *

## Références

- [https://www.atlassian.com/git/tutorials/](https://www.atlassian.com/git/tutorials/)
- [http://git-scm.com/book/fr/v2](http://git-scm.com/book/fr/v2)

* * * * *

Dernière mise à jour : 2016-03-05

